from django.forms import Media
from django.test import TestCase

from pipeline.forms import PipelineFormMedia
from ..utils import pipeline_settings


@pipeline_settings(
    PIPELINE_COLLECTOR_ENABLED=False,
    STYLESHEETS={
        'styles1': {
            'source_filenames': (
                'pipeline/css/first.css',
                'pipeline/css/second.css',
            ),
            'output_filename': 'styles1.min.css',
        },
        'styles2': {
            'source_filenames': (
                'pipeline/css/unicode.css',
            ),
            'output_filename': 'styles2.min.css',
        },
        'print': {
            'source_filenames': (
                'pipeline/css/urls.css',
            ),
            'output_filename': 'print.min.css',
        },
    },
    JAVASCRIPT={
        'scripts1': {
            'source_filenames': (
                'pipeline/js/first.js',
                'pipeline/js/second.js',
            ),
            'output_filename': 'scripts1.min.js',
        },
        'scripts2': {
            'source_filenames': (
                'pipeline/js/application.js',
            ),
            'output_filename': 'scripts2.min.js',
        },
    })
class PipelineFormMediaTests(TestCase):
    """Unit tests for pipeline.forms.PipelineFormMedia."""

    @pipeline_settings(PIPELINE_ENABLED=True)
    def test_css_packages_with_pipeline_enabled(self):
        """Testing PipelineFormMedia.css_packages with PIPELINE_ENABLED=True"""
        class MyMedia(PipelineFormMedia):
            css_packages = {
                'all': ('styles1', 'styles2'),
                'print': ('print',),
            }

            css = {
                'all': ('extra1.css', 'extra2.css')
            }

        media = Media(MyMedia)

        self.assertEqual(
            MyMedia.css,
            {
                'all': [
                    'extra1.css',
                    'extra2.css',
                    '/static/styles1.min.css',
                    '/static/styles2.min.css',
                ],
                'print': ['/static/print.min.css'],
            })
        self.assertEqual(MyMedia.css, media._css)
        expected_regex = [
            r'<link href="%s" type="text/css" media="all" '
            'rel="stylesheet"( /)?>' % path
            for path in (
                '/static/extra1.css',
                '/static/extra2.css',
                '/static/styles1.min.css',
                '/static/styles2.min.css',
            )
        ] + [
            r'<link href="/static/print.min.css" type="text/css" '
            'media="print" rel="stylesheet"( /)?>'
        ]
        for rendered_node, expected_node in zip(
            media.render_css(), expected_regex
        ):
            self.assertRegex(rendered_node, expected_node)

    @pipeline_settings(PIPELINE_ENABLED=False)
    def test_css_packages_with_pipeline_disabled(self):
        """Testing PipelineFormMedia.css_packages with PIPELINE_ENABLED=False"""
        class MyMedia(PipelineFormMedia):
            css_packages = {
                'all': ('styles1', 'styles2'),
                'print': ('print',),
            }

            css = {
                'all': ('extra1.css', 'extra2.css')
            }

        media = Media(MyMedia)

        self.assertEqual(
            MyMedia.css,
            {
                'all': [
                    'extra1.css',
                    'extra2.css',
                    'pipeline/css/first.css',
                    'pipeline/css/second.css',
                    'pipeline/css/unicode.css',
                ],
                'print': ['pipeline/css/urls.css'],
            })
        self.assertEqual(MyMedia.css, media._css)

        expected_regex = [
            '<link href="%s" type="text/css" media="all" '
            'rel="stylesheet"( /)?>' % path
            for path in (
                '/static/extra1.css',
                '/static/extra2.css',
                '/static/pipeline/css/first.css',
                '/static/pipeline/css/second.css',
                '/static/pipeline/css/unicode.css',
            )
        ] + [
            '<link href="/static/pipeline/css/urls.css" type="text/css" '
            'media="print" rel="stylesheet"( /)?>'
        ]
        for rendered_node, expected_node in zip(
            media.render_css(), expected_regex
        ):
            self.assertRegex(rendered_node, expected_node)

    @pipeline_settings(PIPELINE_ENABLED=True)
    def test_js_packages_with_pipeline_enabled(self):
        """Testing PipelineFormMedia.js_packages with PIPELINE_ENABLED=True"""
        class MyMedia(PipelineFormMedia):
            js_packages = ('scripts1', 'scripts2')
            js = ('extra1.js', 'extra2.js')

        media = Media(MyMedia)

        self.assertEqual(
            MyMedia.js,
            [
                'extra1.js',
                'extra2.js',
                '/static/scripts1.min.js',
                '/static/scripts2.min.js',
            ])
        self.assertEqual(MyMedia.js, media._js)
        self.assertEqual(
            media.render_js(),
            [
                '<script type="text/javascript" src="%s"></script>' % path
                for path in (
                    '/static/extra1.js',
                    '/static/extra2.js',
                    '/static/scripts1.min.js',
                    '/static/scripts2.min.js',
                )
            ])

    @pipeline_settings(PIPELINE_ENABLED=False)
    def test_js_packages_with_pipeline_disabled(self):
        """Testing PipelineFormMedia.js_packages with PIPELINE_ENABLED=False"""
        class MyMedia(PipelineFormMedia):
            js_packages = ('scripts1', 'scripts2')
            js = ('extra1.js', 'extra2.js')

        media = Media(MyMedia)

        self.assertEqual(
            MyMedia.js,
            [
                'extra1.js',
                'extra2.js',
                'pipeline/js/first.js',
                'pipeline/js/second.js',
                'pipeline/js/application.js',
            ])
        self.assertEqual(MyMedia.js, media._js)
        self.assertEqual(
            media.render_js(),
            [
                '<script type="text/javascript" src="%s"></script>' % path
                for path in (
                    '/static/extra1.js',
                    '/static/extra2.js',
                    '/static/pipeline/js/first.js',
                    '/static/pipeline/js/second.js',
                    '/static/pipeline/js/application.js',
                )
            ])



import glob
import os
import distutils.spawn


def local_path(path):
    return os.path.join(os.path.dirname(__file__), path)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'TEST_NAME': ':memory:'
    }
}

DEBUG = False

SITE_ID = 1

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sites',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.auth',
    'django.contrib.admin',
    'pipeline',
    'tests.tests'
]


ROOT_URLCONF = 'tests.urls'

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
]

MEDIA_URL = '/media/'

MEDIA_ROOT = local_path('media')

STATICFILES_STORAGE = 'pipeline.storage.PipelineStorage'
STATIC_ROOT = local_path('static/')
STATIC_URL = '/static/'
STATICFILES_DIRS = (
    ('pipeline', local_path('assets/')),
)
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'pipeline.finders.PipelineFinder',
)

SECRET_KEY = "django-pipeline"

PIPELINE = {
    'PIPELINE_ENABLED': True,
    'JS_COMPRESSOR': None,
    'CSS_COMPRESSOR': None,
    'STYLESHEETS': {
        'screen': {
            'source_filenames': (
                'pipeline/css/first.css',
                'pipeline/css/second.css',
                'pipeline/css/urls.css'
            ),
            'output_filename': 'screen.css'
        },
        'screen_media': {
            'source_filenames': (
                'pipeline/css/first.css',
                'pipeline/css/second.css',
                'pipeline/css/urls.css'
            ),
            'output_filename': 'screen_media.css',
            'extra_context': {
                'media': 'screen and (min-width:500px)',
            },
        },
        'screen_title': {
            'source_filenames': (
                'pipeline/css/first.css',
                'pipeline/css/second.css',
                'pipeline/css/urls.css'
            ),
            'output_filename': 'screen_title.css',
            'extra_context': {
                'title': 'Default Style',
            },
        }
    },
    'JAVASCRIPT': {
        'scripts': {
            'source_filenames': (
                'pipeline/js/first.js',
                'pipeline/js/second.js',
                'pipeline/js/application.js',
                'pipeline/templates/**/*.jst'
            ),
            'output_filename': 'scripts.js'
        },
        'scripts_async': {
            'source_filenames': (
                'pipeline/js/first.js',
                'pipeline/js/second.js',
                'pipeline/js/application.js',
                'pipeline/templates/**/*.jst'
            ),
            'output_filename': 'scripts_async.js',
            'extra_context': {
                'async': True,
            }
        },
        'scripts_defer': {
            'source_filenames': (
                'pipeline/js/first.js',
                'pipeline/js/second.js',
                'pipeline/js/application.js',
                'pipeline/templates/**/*.jst'
            ),
            'output_filename': 'scripts_defer.js',
            'extra_context': {
                'defer': True,
            }
        },
        'scripts_async_defer': {
            'source_filenames': (
                'pipeline/js/first.js',
                'pipeline/js/second.js',
                'pipeline/js/application.js',
                'pipeline/templates/**/*.jst'
            ),
            'output_filename': 'scripts_async_defer.js',
            'extra_context': {
                'async': True,
                'defer': True,
            }
        }
    }
}

NODE_MODULES_PATH = local_path('../node_modules')
NODE_BIN_PATH = os.path.join(NODE_MODULES_PATH, '.bin')
NODE_EXE_PATH = distutils.spawn.find_executable('node')
JAVA_EXE_PATH = distutils.spawn.find_executable('java')
CSSTIDY_EXE_PATH = distutils.spawn.find_executable('csstidy')
HAS_NODE = bool(NODE_EXE_PATH)
HAS_JAVA = bool(JAVA_EXE_PATH)
HAS_CSSTIDY = bool(CSSTIDY_EXE_PATH)

if HAS_NODE:
    def node_exe_path(command):
        exe_ext = '.cmd' if os.name == 'nt' else ''
        return os.path.join(NODE_BIN_PATH, "%s%s" % (command, exe_ext))

    PIPELINE.update({
        'SASS_BINARY': node_exe_path('node-sass'),
        'COFFEE_SCRIPT_BINARY': node_exe_path('coffee'),
        'COFFEE_SCRIPT_ARGUMENTS': ['--no-header'],
        'LESS_BINARY': node_exe_path('lessc'),
        'BABEL_BINARY': node_exe_path('babel'),
        'BABEL_ARGUMENTS': ['--presets', 'es2015'],
        'STYLUS_BINARY': node_exe_path('stylus'),
        'LIVE_SCRIPT_BINARY': node_exe_path('lsc'),
        'LIVE_SCRIPT_ARGUMENTS': ['--no-header'],
        'YUGLIFY_BINARY': node_exe_path('yuglify'),
        'UGLIFYJS_BINARY': node_exe_path('uglifyjs'),
        'CSSMIN_BINARY': node_exe_path('cssmin'),
    })

if HAS_NODE and HAS_JAVA:
    PIPELINE.update({
        'CLOSURE_BINARY': [
            JAVA_EXE_PATH, '-jar',
            os.path.join(NODE_MODULES_PATH, 'google-closure-compiler-java', 'compiler.jar')],
        'YUI_BINARY': [
            JAVA_EXE_PATH, '-jar',
            glob.glob(os.path.join(NODE_MODULES_PATH, 'yuicompressor', 'build', '*.jar'))[0]]
    })

if HAS_CSSTIDY:
    PIPELINE.update({'CSSTIDY_BINARY': CSSTIDY_EXE_PATH})

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'DIRS': [local_path('templates')],
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ]
        }
    },
    {
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'APP_DIRS': True,
        'DIRS': [local_path('templates')],
        'OPTIONS': {
            'extensions': ['pipeline.jinja2.PipelineExtension']
        }
    }
]

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'pipeline.templatetags.pipeline': {
            'handlers': ['console'],
            'level': 'ERROR',
        },
    },
}



from django.conf.urls import url
from django.contrib import admin
from django.views.generic import TemplateView


urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name="index.html"), name="index"),
    url(r'^empty/$', TemplateView.as_view(template_name="empty.html"), name="empty"),
    url(r'^admin/', admin.site.urls),
]
